# Dictionary Driven Checker 🔍

Validates data against dictionary specifications using plugin-based validation rules. Supports numeric, categorical, date, and text validation with customizable rules and comprehensive reporting.

---

📅 **Build Date:** [INSERT_DATE_HERE]

This package was last updated on the date above.  
For reproducibility and support, always refer to this date when sharing logs or output.

---

## 📦 Project Structure

```
dictionary_driven_checker/
├── __init__.py         # Package interface and version info
├── main.py            # CLI entry point
├── utils.py           # Helper functions
├── env.py             # Environment detection
├── plugins/           # Validation plugins
│   ├── __init__.py    # Plugin registry
│   ├── numeric_plugin.py
│   ├── text_plugin.py
│   ├── date_plugin.py
│   └── validators.py
└── README.md         # This documentation
```

---

## 🚀 Usage (Development)

### Command Line
```bash
python -m scriptcraft.tools.dictionary_driven_checker --data-file data.csv --dictionary-file dict.csv --output-dir output
```

### Python API
```python
from scriptcraft.tools.dictionary_driven_checker import DictionaryDrivenChecker

checker = DictionaryDrivenChecker()
checker.run(
    data_file="data.csv",
    dictionary_file="dict.csv",
    output_dir="output"
)
```

Arguments:
- `--data-file`: Path to data file to validate
- `--dictionary-file`: Path to dictionary specification file
- `--output-dir`: Output directory for validation reports
- `--domain`: Optional domain context for validation
- `--plugins`: Specific validation plugins to use
- `--strict`: Enable strict validation mode

---

## ⚙️ Features

- 🔍 Plugin-based validation system
- 📊 Multiple validation types (numeric, categorical, date, text)
- 🎯 Dictionary-driven validation rules
- 📋 Comprehensive validation reports
- 🔄 Batch processing support
- 🛡️ Error handling and recovery
- 📈 Performance metrics and logging
- 🎨 Customizable validation rules
- 🔧 Extensible plugin architecture

---

## 🔧 Dev Tips

- Use domain-specific validation plugins for healthcare data
- Test validation rules with sample data before processing large files
- Check plugin compatibility with your dictionary format
- Review validation reports for rule effectiveness
- Customize validation thresholds based on data quality requirements
- Use batch processing for multiple data files

---

## 🧪 Testing

### Unit Tests
```bash
python -m pytest tests/tools/test_dictionary_driven_checker.py
```

### Integration Tests
```bash
python -m pytest tests/integration/tools/test_dictionary_driven_checker_integration.py
```

### Test Data
Example files needed:
- Sample data files with various formats
- Dictionary specification files
- Expected validation reports
- Test cases for different validation types
- Plugin-specific test data

---

## 🔄 Dependencies

Required packages:
- pandas >= 1.3.0
- numpy >= 1.20.0
- openpyxl >= 3.0.0
- Python >= 3.8

System requirements:
- Memory: 200MB base + 100MB per file
- Storage: 500MB for processing and output
- CPU: Multi-core recommended for batch processing

---

## 🚨 Error Handling

Common errors and solutions:
1. **Validation Plugin Error**
   - Cause: Plugin not found or incompatible
   - Solution: Check plugin installation and compatibility
2. **Dictionary Format Error**
   - Cause: Dictionary file format not recognized
   - Solution: Verify dictionary format and required columns
3. **Data Validation Error**
   - Cause: Data format incompatible with validation rules
   - Solution: Check data format and validation rule compatibility

---

## 📊 Performance

Expectations:
- Processing speed: 1000-5000 records per second
- Memory usage: 200MB base + 100MB per file
- File size limits: Up to 500MB per data file

Optimization tips:
- Use specific plugins instead of all plugins
- Process large files in chunks
- Enable parallel processing for multiple files
- Optimize validation rule patterns

---

## 📋 Development Checklist

### 1. File Structure ✅
- [x] Standard package layout
  - [x] __init__.py with version info
  - [x] main.py for CLI
  - [x] utils.py for helpers
  - [x] env.py for environment detection
  - [x] plugins/ directory
  - [x] README.md
- [x] Clean organization
- [x] No deprecated files

### 2. Documentation ✅
- [x] Version information
- [x] Package-level docstring
- [x] Function docstrings
- [x] Type hints
- [x] README.md
- [x] API documentation
- [x] Error code reference
- [x] Troubleshooting guide

### 3. Code Implementation ✅
- [x] Core functionality
- [x] CLI interface
- [x] Error handling
- [x] Input validation
- [x] Type checking
- [x] Performance optimization
- [x] Security considerations

### 4. Testing ⬜
- [ ] Unit tests
- [ ] Integration tests
- [ ] Performance tests
- [ ] Edge case tests
- [ ] Error condition tests
- [ ] Test data examples

### 5. Error Handling ✅
- [x] Custom exceptions
- [x] Error messages
- [x] Error logging
- [x] Error recovery
- [x] Input validation

### 6. Performance ✅
- [x] Large dataset testing
- [x] Memory optimization
- [x] Progress reporting
- [x] Chunked processing
- [x] Performance metrics

### 7. Configuration ✅
- [x] Command-line arguments
- [x] Configuration validation
- [x] Environment variables
- [x] Default settings
- [x] Documentation

### 8. Packaging ✅
- [x] Dependencies specified
- [x] Version information
- [x] Package structure
- [x] Installation tested
- [x] Distribution tested

---

## 📋 Current Status and Future Improvements

### ✅ Completed Items
1. **Core Implementation**
   - Plugin-based validation system
   - Multiple validation types
   - Dictionary-driven rules
   - Comprehensive reporting
   - Batch processing support

2. **Documentation**
   - Main README structure
   - Usage examples
   - Error handling guide
   - Performance metrics

3. **Infrastructure**
   - Environment detection
   - CLI integration
   - Error handling
   - Configuration management

### 🔄 Partially Complete
1. **Testing**
   - ✅ Basic structure
   - ❌ Need comprehensive test suite
   - ❌ Need integration tests
   - ❌ Need performance tests

2. **Features**
   - ✅ Basic validation system
   - ❌ Need advanced validation rules
   - ❌ Need custom plugin support
   - ❌ Need validation rule learning

### 🎯 Prioritized Improvements

#### High Priority
1. **Testing Enhancement**
   - Add comprehensive test suite
   - Create integration tests
   - Add performance benchmarks
   - Improve error case coverage

2. **Feature Enhancement**
   - Add advanced validation rules
   - Implement custom plugin support
   - Add validation rule learning
   - Improve plugin architecture

#### Medium Priority
3. **Documentation**
   - Add detailed API docs
   - Create troubleshooting guide
   - Add performance tuning guide
   - Document common patterns

4. **User Experience**
   - Add progress tracking
   - Improve error messages
   - Add configuration validation
   - Create interactive mode

#### Low Priority
5. **Advanced Features**
   - Add ML-based validation
   - Support more data formats
   - Add validation rule optimization
   - Create validation summaries

6. **Development Tools**
   - Add development utilities
   - Create debugging helpers
   - Add profiling support
   - Improve error messages

---

## 🤝 Contributing

1. Branch naming: `feature/dictionary-driven-checker-[feature]`
2. Required for all changes:
   - Unit tests
   - Documentation updates
   - Checklist review
3. Code review process in CONTRIBUTING.md 